use color_eyre::eyre;
use eyre::eyre;

use crate::lib::Vec2;


#[derive(Debug)]
struct Cell {
    height: u32,
    min: bool,
}


fn load(data: &str) -> eyre::Result<Vec2<Cell>> {
    let num_columns = data.lines().next().unwrap().len();
    let mut heights = Vec2::new(num_columns);
    for line in data.lines() {
        let mut row = Vec::new();
        for c in line.chars() {
            match c.to_digit(10) {
                Some(d) => row.push(Cell {height: d, min: true}),
                None => return Err(eyre!("Invalid character (not a digit): {}", c)),
            }
        }
        heights.push(row);
    }
    Ok(heights)
}


fn scan(map: &mut Vec2<Cell>) {
    for r in 0..map.row_count() {
        for col in 0..map.col_count() {
            if col > 0 {
                if map[r][col].height <= map[r][col - 1].height {
                    map[r][col - 1].min = false;
                    if map[r][col].height == map[r][col - 1].height {
                        map[r][col].min = false;
                    }
                }
            }

            if r > 0 {
                if map[r][col].height <= map[r - 1][col].height {
                    map[r - 1][col].min = false;
                    if map[r][col].height == map[r - 1][col].height {
                        map[r][col].min = false;
                    }
                }
            }
        }
    }

    // for (r, row) in map.rows().enumerate() {
    //     for (col, cell) in row.iter().enumerate() {
    //         if col > 0 && cell.height < row[col - 1].height {
    //             map[r][col - 1].min = false;
    //         }

    //         if r > 0 && cell.height < map[r - 1][col].height {
    //             map[r - 1][col].min = false;
    //         }
    //     }
    // }
}


fn part1(map: &Vec2<Cell>) -> u32 {
    map.values()
        .filter(|c| c.min)
        .fold(0, |acc, c| acc + c.height + 1)
}


pub fn run(data: &str) -> eyre::Result<(u32, u32)> {
    let sample = "2199943210\n3987894921\n9856789892\n8767896789\n9899965678";
    let mut map = load(sample)?;
    scan(&mut map);

    dbg!(&map[1], &map[2]);

    for (i, cell) in map.values().enumerate() {
        if cell.min {
            println!("{}, {} ({})", i / map.row_count(), i % map.row_count(), cell.height)
        }
    }

    let one = part1(&map);
    Ok((one, 0))
}
